@@ -4,9 +4,9 @@
 if [ ! -d "$FILEGDB" ]; then
   mkdir -p $FILEGDB
   cd $FILEGDB
-  wget -q https://github.com/Esri/file-geodatabase-api/raw/master/FileGDB_API_1.5.1/FileGDB_API_1_5_1-64gcc51.tar.gz
-  tar -xzf FileGDB_API_1_5_1-64gcc51.tar.gz --strip=1 FileGDB_API-64gcc51
-  rm FileGDB_API_1_5_1-64gcc51.tar.gz
+  wget -q https://github.com/Esri/file-geodatabase-api/raw/master/FileGDB_API_1.5.2/FileGDB_API-RHEL7-64gcc83.tar.gz
+  tar -xzf FileGDB_API-RHEL7-64gcc83.tar.gz --strip=1 FileGDB_API-RHEL7-64gcc83
+  rm FileGDB_API-RHEL7-64gcc83.tar.gz
   rm -rf samples
   rm -rf doc
 fi
