@@ -25,8 +25,8 @@ def test_dump_layer(path_gpx):
 
 
 def test_dump_layer_vfs(path_coutwildrnp_zip):
-    path = 'zip://{}'.format(path_coutwildrnp_zip)
-    result = CliRunner().invoke(main_group, ['dump', path])
+    path = "zip://{}".format(path_coutwildrnp_zip)
+    result = CliRunner().invoke(main_group, ["dump", path])
     assert result.exit_code == 0
     loaded = json.loads(result.output)
     with fiona.open(path) as src:
